package uk.co.maxant.test;

import java.sql.Timestamp;
import java.util.Date;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

/**
 * Session Bean implementation class EJBSeven
 */
@DeclareRoles({Roles.REGISTERED})
@Stateless
@TransactionManagement(TransactionManagementType.CONTAINER)
public class EJBSeven implements EJBSevenLocal {

	@Resource(mappedName = "jms/destinationResource")
	private Queue q;

	@Resource(mappedName = "jmsConnectionPool")
	private QueueConnectionFactory qcf;

	@Resource 
	private SessionContext sessionContext;

	@PersistenceContext(unitName="MessageManagement")
	private EntityManager em;
	
	private QueueConnection qc = null;
	private QueueSession qs = null;
	private QueueSender qSender = null;
	
	@PostConstruct
	protected void init(){
		try {
			qc = qcf.createQueueConnection();
			qs = qc.createQueueSession(true, Session.SESSION_TRANSACTED);
			qSender = qs.createSender(q);
		} catch (JMSException e) {
			throw new RuntimeException("unable to create ejb", e);
		}
	}
	
	@PreDestroy
	protected void close(){
		if (qc != null) {
			try {
				qc.close();
			} catch (JMSException e) {
				throw new RuntimeException("unable to destroy ejb", e);
			}
		}
	}

	/**
	 * Default constructor.
	 */
	public EJBSeven() {
	}

	@Override
	@RolesAllowed({Roles.REGISTERED})
	@TransactionAttribute(TransactionAttributeType.REQUIRED)
	public String execute(String command) {

		// send a message to a Queue, for processing async

		String ret = "hello " + sessionContext.getCallerPrincipal() + ", you said: " + command;
		try {
			addMessageToDB();
			
			sendAsyncMessage(command);

			if(command.equals("fdsa")){
				sessionContext.setRollbackOnly();
//				throw new RuntimeException();
			}
			
		} catch (JMSException e) {
			ret = "FAILED";
		}

		return ret;
	}

	/**
	 * writes a message to the database.
	 */
	private void addMessageToDB() {
		
		Message msg = new Message(
				String.valueOf(sessionContext.getCallerPrincipal()), 
				new Timestamp(System.currentTimeMillis()), 
				Status.NEW.toString());
		em.persist(msg);
	}

	/**
	 * sends a JMS message with the content
	 */
	private void sendAsyncMessage(String content) throws JMSException {
		TextMessage msg = qs.createTextMessage();
		msg.setText("This is a message from " + 
				sessionContext.getCallerPrincipal() + " sent at " + new Date() + ": " + content);
		msg.setStringProperty(MDBean.FROM, sessionContext.getCallerPrincipal().getName());
		qSender.send(msg);
	}

}
